﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace howto_preview_textbox_changes
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Attach the TextBox's ContextMenuStrip.
        private void Form1_Load(object sender, EventArgs e)
        {
            txtInteger.ContextMenuStrip = ctxTextBox;
        }

        // Possible change types for the TextBox's value.
        private enum EditType
        {
            NewCharacter,
            Cut,
            Paste,
            Delete,
            Backspace,
        }

        // Handle normal characters.
        private void txtInteger_KeyPress(object sender, KeyPressEventArgs e)
        {
            // Do nothing for special characters.
            if ((e.KeyChar < ' ') || (e.KeyChar > '~')) return;

            // See if the change is valid.
            e.Handled = ShouldCancelTextBoxEvent(
                txtInteger, EditType.NewCharacter, e.KeyChar, false);
        }

        // Handle Ctrl+A, Ctrl+X, Ctrl+V, Shift+Insert,
        // Shift+Delete, Delete, and Backspace.
        private void txtInteger_KeyDown(object sender, KeyEventArgs e)
        {
            // Look for the necessary key combinations.
            bool cancel_event;
            if (e.Control && (e.KeyCode == Keys.A))
            {
                // Handle this one just for convenience.
                txtInteger.Select(0, txtInteger.Text.Length);
                cancel_event = true;
            }
            else if (e.Control && (e.KeyCode == Keys.X))
            {
                cancel_event = ShouldCancelTextBoxEvent(txtInteger, EditType.Cut, ' ', false);
            }
            else if (e.Control && (e.KeyCode == Keys.V))
            {
                cancel_event = ShouldCancelTextBoxEvent(txtInteger, EditType.Paste, ' ', false);
            }
            else if (e.Shift && (e.KeyCode == Keys.Insert))
            {
                cancel_event = ShouldCancelTextBoxEvent(txtInteger, EditType.Paste, ' ', false);
            }
            else if (e.Shift && (e.KeyCode == Keys.Delete))
            {
                cancel_event = ShouldCancelTextBoxEvent(txtInteger, EditType.Cut, ' ', false);
            }
            else if (e.KeyCode == Keys.Delete)
            {
                cancel_event = ShouldCancelTextBoxEvent(txtInteger, EditType.Delete, ' ', false);
            }
            else if (e.KeyCode == Keys.Back)
            {
                cancel_event = ShouldCancelTextBoxEvent(txtInteger, EditType.Backspace, ' ', false);
            }
            else
            {
                // We didn't handle the event.
                // Let the event proceed normally.
                cancel_event = false;
            }

            // If we handled it, stop the event.
            if (cancel_event)
            {
                e.Handled = true;
                e.SuppressKeyPress = true;
            }
        }

        // Return true if the value is a valid integer.
        private bool ValueIsValid(string text_value)
        {
            // Do not allow spaces.
            if (text_value.Contains(' ')) return false;

            // Allow a blank string, -, and +.
            if ((text_value.Length == 0) ||
                (text_value == "-") ||
                (text_value == "+")) return true;

            // See if the text parses.
            int test_value;
            return int.TryParse(text_value, out test_value);
        }

        // Compose the TextBox's new value and call ValueIsValid
        // to see if it is valid.
        //
        // If the change is invalid, beep and return true to indicate
        // the event should be canceled.
        //
        // If the change is valid and assign_value is true, assign the
        // value to the TextBox and then return true to indicate that
        // the event is no longer needed.
        //
        // If the change is valid and assign_value is false, then return
        // false to indicate that the event still needs to be handled.
        private bool ShouldCancelTextBoxEvent(
            TextBox txt, EditType edit_type, char new_char, bool assign_value)
        {
            // Get the new value.
            int selection_start;
            string new_value =
                GetNewTextBoxValue(txt, edit_type, new_char, out selection_start);

            // See if it's a valid value.
            if (ValueIsValid(new_value))
            {
                // It's okay. Accept it.
                if (assign_value)
                {
                    txt.Text = new_value;
                    txt.Select(selection_start, 0);
                    return true;
                }
                return false;
            }
            else
            {
                // The new value is invalid. Complain.
                System.Media.SystemSounds.Beep.Play();
                return true;
            }
        }

        // Return the value that the TextBox will have if this change is allowed.
        private string GetNewTextBoxValue(TextBox txt, EditType edit_type,
            char new_char, out int selection_start)
        {
            // Get the pieces of the current text.
            selection_start = txt.SelectionStart;
            string current_text = txt.Text;
            string left_text = current_text.Substring(0, selection_start);
            string selected_text = txt.SelectedText;
            string right_text =
                current_text.Substring(selection_start + txt.SelectionLength);

            // Compose the result.
            string result = "";
            switch (edit_type)
            {
                case EditType.NewCharacter:
                    result = left_text + new_char + right_text;
                    selection_start++;
                    break;
                case EditType.Cut:
                    result = left_text + right_text;
                    if (txt.SelectionLength > 0)
                    {
                        Clipboard.Clear();
                        Clipboard.SetText(selected_text);
                    }
                    break;
                case EditType.Paste:
                    if (Clipboard.ContainsText())
                    {
                        selected_text = Clipboard.GetText();
                        selection_start += selected_text.Length;
                    }
                    result = left_text + selected_text + right_text;
                    break;
                case EditType.Delete:
                    if (selected_text.Length == 0)
                    {
                        // Remove the following character.
                        if (right_text.Length > 0)
                            right_text = right_text.Substring(1);
                    }
                    else
                    {
                        // Remove the selected text.
                        selected_text = "";
                    }
                    result = left_text + selected_text + right_text;
                    break;
                case EditType.Backspace:
                    if (selected_text.Length == 0)
                    {
                        // Remove the previous character.
                        if (left_text.Length > 0)
                        {
                            left_text = left_text.Substring(0, selection_start - 1);
                            selection_start--;
                        }
                    }
                    else
                    {
                        // Remove the selected text.
                        selected_text = "";
                    }
                    result = left_text + selected_text + right_text;
                    break;
            }

            // Return the result.
            return result;
        }

        // Enable appropriate context menu items.
        private void ctxTextBox_Opening(object sender, CancelEventArgs e)
        {
            ctxUndo.Enabled = txtInteger.CanUndo;

            // Copy and Cut are enabled if anything is selected.
            ctxCopy.Enabled = (txtInteger.SelectionLength > 0);
            ctxCut.Enabled = (txtInteger.SelectionLength > 0);

            // Delete is enabled if anything is selected or there
            // is a character after the insertion point to delete.
            ctxDelete.Enabled =
                ((txtInteger.SelectionLength > 0) ||
                 (txtInteger.SelectionStart < txtInteger.Text.Length));

            // Paste is enabled if the clipboard contains text.
            ctxPaste.Enabled = Clipboard.ContainsText();
        }

        // Make context menu items to do the
        // same work as the control characters.
        private void ctxUndo_Click(object sender, EventArgs e)
        {
            txtInteger.Undo();
        }
        private void ctxCopy_Click(object sender, EventArgs e)
        {
            Clipboard.Clear();
            Clipboard.SetText(txtInteger.SelectedText);
        }
        private void ctxCut_Click(object sender, EventArgs e)
        {
            ShouldCancelTextBoxEvent(txtInteger, EditType.Cut, ' ', true);
        }
        private void ctxPaste_Click(object sender, EventArgs e)
        {
            ShouldCancelTextBoxEvent(txtInteger, EditType.Paste, ' ', true);
        }
        private void ctxDelete_Click(object sender, EventArgs e)
        {
            ShouldCancelTextBoxEvent(txtInteger, EditType.Delete, ' ', true);
        }
    }
}
